@@ -7,6 +7,7 @@
 # file LICENSE or http://www.opensource.org/licenses/mit-license.php.
 #
 
+import calendar
 import numbers
 from struct import pack, unpack
 
@@ -31,9 +32,9 @@ __all__ = [
 
 # ===----------------------------------------------------------------------===
 
-
 ChainParameters = recordtype('ChainParameters',
-    ['magic', 'port', 'genesis', 'testnet', 'max_value', 'transient_reward',
+    ['magic', 'port', 'genesis', 'testnet', 'pubkey_hash_prefix',
+     'script_hash_prefix', 'secret_prefix', 'max_value', 'transient_reward',
      'transient_budget', 'perpetual_reward', 'perpetual_budget', 'fee_budget',
      'maximum_target', 'next_target', 'alert_keys','checkpoints', 'features'])
 
@@ -79,13 +80,13 @@ class Output(SerializableMixin):
 # ===----------------------------------------------------------------------===
 
 class Input(SerializableMixin):
-    def __init__(self, output_hash=0, output_index=0xffffffff,
-                 endorsement=None, sequence=0xffffffff, *args, **kwargs):
+    def __init__(self, hash=0, index=0xffffffff, endorsement=None,
+                 sequence=0xffffffff, *args, **kwargs):
         if endorsement is None:
             endorsement = kwargs.pop('coinbase', self.get_script_class()())
         super(Input, self).__init__(*args, **kwargs)
-        self.output_hash = output_hash
-        self.output_index = output_index
+        self.hash = hash
+        self.index = index
         self.endorsement = endorsement
         self.sequence = sequence
 
@@ -94,8 +95,8 @@ class Input(SerializableMixin):
         return getattr(cls, 'script_class', Script)
 
     def serialize(self):
-        result  = serialize_hash(self.output_hash, 32)
-        result += pack('<I', self.output_index)
+        result  = serialize_hash(self.hash, 32)
+        result += pack('<I', self.index)
         if hasattr(self.endorsement, 'serialize'):
             result += self.endorsement.serialize()
         else:
@@ -105,12 +106,12 @@ class Input(SerializableMixin):
     @classmethod
     def deserialize(cls, file_):
         initargs = {}
-        initargs['output_hash'] = deserialize_hash(file_, 32)
-        initargs['output_index'] = unpack('<I', file_.read(4))[0]
+        initargs['hash'] = deserialize_hash(file_, 32)
+        initargs['index'] = unpack('<I', file_.read(4))[0]
         str_ = deserialize_varchar(file_) # <-- might be coinbase!
         initargs['sequence'] = unpack('<I', file_.read(4))[0]
-        if not all((initargs['output_hash']  == 0,
-                    initargs['output_index'] == 0xffffffff)):
+        if not all((initargs['hash']  == 0,
+                    initargs['index'] == 0xffffffff)):
             initargs['coinbase'] = str_
         else:
             initargs['endorsement'] = cls.get_script_class().deserialize(
@@ -119,25 +120,25 @@ class Input(SerializableMixin):
 
     @property
     def is_coinbase(self):
-        return all((self.output_hash  == 0,
-                    self.output_index == 0xffffffff))
+        return all((self.hash  == 0,
+                    self.index == 0xffffffff))
 
     def __eq__(self, other):
-        return all((self.output_hash  == other.output_hash,
-                    self.output_index == other.output_index,
-                    self.endorsement  == other.endorsement,
-                    self.sequence     == other.sequence))
+        return all((self.hash        == other.hash,
+                    self.index       == other.index,
+                    self.endorsement == other.endorsement,
+                    self.sequence    == other.sequence))
 
     def __repr__(self):
         sequence_str = (self.sequence!=0xffffffff
             and ', sequence=%d' % self.sequence
              or '')
-        return ('%s(output_hash=0x%064x, '
-                   'output_index=%d, '
+        return ('%s(hash=0x%064x, '
+                   'index=%d, '
                    '%s=%s%s)') % (
             self.__class__.__name__,
-            self.output_hash,
-            self.output_index==0xffffffff and -1 or self.output_index,
+            self.hash,
+            self.index==0xffffffff and -1 or self.index,
             self.is_coinbase and 'coinbase' or 'endorsement',
             repr(self.endorsement),
             sequence_str)
@@ -151,9 +152,9 @@ class Transaction(SerializableMixin, HashableMixin):
         if outputs is None: outputs = ()
         super(Transaction, self).__init__(*args, **kwargs)
         self.version = version
-        getattr(self, 'inputs_create', lambda x:setattr(x, 'inputs', list()))(self)
+        getattr(self, 'inputs_create', lambda:setattr(x, 'inputs', list()))
         self.inputs.extend(inputs)
-        getattr(self, 'outputs_create', lambda x:setattr(x, 'outputs', list()))(self)
+        getattr(self, 'outputs_create', lambda:setattr(x, 'outputs', list()))
         self.outputs.extend(outputs)
         self.lock_time = lock_time
         self.reference_height = reference_height
@@ -258,7 +259,11 @@ class Block(SerializableMixin, HashableMixin):
         result  = pack('<I', self.version)
         result += serialize_hash(self.parent_hash, 32)
         result += serialize_hash(self.merkle_hash, 32)
-        result += pack('<I', self.time)
+        if isinstance(self.time, numbers.Integral):
+            time = self.time
+        else:
+            time = calendar.timegm(self.time.utctimetuple())
+        result += pack('<I', time)
         result += pack('<I', self.bits)
         result += pack('<I', self.nonce)
         return result
