@@ -19,13 +19,19 @@ from bitcoin.serialize import (
 
 from .tools import Bits, StringIO, icmp, list, tuple
 
+__all__ = (
+    'PatriciaLink',
+    'BasePatriciaDict',
+    'MemoryPatriciaDict',
+)
+
 SENTINAL = object()
 
 # ===----------------------------------------------------------------------===
 
 import numbers
 
-class PatriciaLink(SerializableMixin, HashableMixin):
+class PatriciaLink(HashableMixin):
     __slots__ = 'prefix node _hash'.split()
 
     def __init__(self, prefix, node=None, hash=None, *args, **kwargs):
@@ -43,28 +49,6 @@ class PatriciaLink(SerializableMixin, HashableMixin):
     def pruned(self):
         return self.node is None
 
-    def serialize(self, digest=False):
-        result = serialize_varchar(self.prefix)
-        if digest:
-            result += serialize_hash(self.hash, 32)
-        else:
-            result += serialize_varchar(self.node.serialize(digest=digest))
-        return result
-    @classmethod
-    def deserialize_node(cls, file_, *args, **kwargs):
-        node_class = getattr(self, 'get_node_class', lambda:
-                     getattr(self, 'node_class', PatriciaNode))()
-        return node_class.deserialize(file_, *args, **kwargs)
-    @classmethod
-    def deserialize(cls, file_, digest=False, *args, **kwargs):
-        initargs = {}
-        initargs['prefix'] = deserialize_varchar(file_)
-        if digest:
-            initargs['hash'] = deserialize_hash(file_, 32)
-        else:
-            initargs['node'] = cls.deserialize_node(file_, digest=digest, *args, **kwargs)
-        return cls(**initargs)
-
     def hash__getter(self):
         if getattr(self, '_hash', None) is not None:
             return self._hash
@@ -105,9 +89,9 @@ import operator
 from bisect import bisect_left
 from os.path import commonprefix
 from struct import pack, unpack
-class PatriciaNode(SerializableMixin, HashableMixin):
-    """An ordered dictionary implemented with a hybrid level- and node-
-    compressed prefix tree."""
+class BasePatriciaDict(SerializableMixin, HashableMixin):
+    """An ordered dictionary implemented with a hybrid
+    level- and node-compressed prefix tree."""
     __slots__ = 'value children _hash size length'.split()
 
     OFFSET_LEFT  = 0
@@ -191,7 +175,7 @@ class PatriciaNode(SerializableMixin, HashableMixin):
             assert 0 < len(link.prefix) < 2**64
         assert len(children) == len(set(l.prefix[0] for l in children))
 
-        super(PatriciaNode, self).__init__(*args, **kwargs)
+        super(BasePatriciaDict, self).__init__(*args, **kwargs)
 
         self.value, self.prune_value = value, prune_value
         getattr(self, 'children_create', lambda:setattr(self, 'children', list()))()
@@ -235,7 +219,7 @@ class PatriciaNode(SerializableMixin, HashableMixin):
         return flags
 
     def serialize(self, digest=False):
-        def _serialize_link(link):
+        def _serialize_branch(link):
             len_ = len(link.prefix)
             if len_ >= 3:
                 parts.append(serialize_varint(len_-3))
@@ -250,9 +234,9 @@ class PatriciaNode(SerializableMixin, HashableMixin):
             flags &= self.HASH_MASK
         parts.append(pack('B', flags))
         if self.left is not None:
-            _serialize_link(self.left)
+            _serialize_branch(self.left)
         if self.right is not None:
-            _serialize_link(self.right)
+            _serialize_branch(self.right)
         if self.value is not None:
             parts.append(serialize_varchar(self.value))
         return b''.join(parts)
@@ -284,7 +268,7 @@ class PatriciaNode(SerializableMixin, HashableMixin):
         return cls(**initargs)
 
     def hash__getter(self):
-        return super(PatriciaNode, self).hash__getter(digest=True)
+        return super(BasePatriciaDict, self).hash__getter(digest=True)
 
     def __hash__(self):
         "x.__hash__() <==> hash(x)"
@@ -309,7 +293,7 @@ class PatriciaNode(SerializableMixin, HashableMixin):
             def _hash():
                 return getattr(other, 'hash', NotImplemented)
             def _icmp():
-                if isinstance(other, PatriciaNode):
+                if isinstance(other, BasePatriciaDict):
                     return icmp(self.iteritems(), other.iteritems())
                 elif hasattr(other, 'keys'):
                     return icmp(self.iteritems(), sorted((x,other[x]) for x in other))
@@ -355,7 +339,7 @@ class PatriciaNode(SerializableMixin, HashableMixin):
 
     def _forward_iterator(self):
         "Returns a forward iterator over the trie"
-        path = [(self, 0, b'')]
+        path = [(self, 0, Bits())]
         while path:
             node, idx, prefix = path.pop()
             if idx==0 and node.value is not None:
@@ -369,7 +353,7 @@ class PatriciaNode(SerializableMixin, HashableMixin):
 
     def _reverse_iterator(self):
         "Returns a reverse/backwards iterator over the trie"
-        path = [(self, len(self.children)-1, b'')]
+        path = [(self, len(self.children)-1, Bits())]
         while path:
             node, idx, prefix = path.pop()
             if idx<0 and node.value is not None:
@@ -432,8 +416,8 @@ class PatriciaNode(SerializableMixin, HashableMixin):
         """Returns the 2-tuple (prefix, node) where node either contains the value
         corresponding to the key, or is the most specific prefix on the path which
         would contain the key if it were there. The key was found if prefix==key
-        and the HAS_VALUE bit of node.flags is set."""
-        prefix, subkey, node = b'', key, self
+        and the node.value is not None."""
+        prefix, subkey, node = Bits(), key, self
         while prefix != key:
             for idx,link in enumerate(node.children):
                 if subkey.startswith(link.prefix):
@@ -519,12 +503,12 @@ class PatriciaNode(SerializableMixin, HashableMixin):
             # of the child links, if they have a prefix in common, or if
             # matches any existing child of the node or not.
             remaining_key = key[len(prefix):]
-            idx = bisect_left(old_node.children, link_class(prefix=remaining_key[:1], node=None))
+            idx = bisect_left(old_node.children, link_class(prefix=remaining_key[:1]))
 
             if idx in xrange(len(old_node.children)):
                 common_prefix = commonprefix([old_node.children[idx].prefix, remaining_key])
             else:
-                common_prefix = ''
+                common_prefix = Bits()
 
             if common_prefix == remaining_key:
                 inner_node = node_class(
@@ -642,10 +626,17 @@ class PatriciaNode(SerializableMixin, HashableMixin):
         "x.__delitem__(y) <==> del x[y]"
         self.delete([key])
 
-    def copy(self):
-        node_class = getattr(self, 'get_node_class',
-            lambda: getattr(self, 'node_class', self.__class__))()
-        return node_class(value=self.value, children=self.children)
+    def copy(self, node_class=None):
+        if node_class is None:
+            node_class = getattr(self, 'get_node_class',
+                lambda: getattr(self, 'node_class', self.__class__))()
+        return node_class(
+            value       = self.value,
+            children    = self.children,
+            prune_value = self.prune_value)
+
+class MemoryPatriciaDict(BasePatriciaDict):
+    pass
 
 #
 # End of File
